#ifndef SYSTEM_HEADER_H
#define SYSTEM_HEADER_H

#define SYSTEM_MACRO(m) int m = 0

namespace SYSTEM_NS {
class structure {
  int member;
};

float global;

void function() {
}
}

#endif // SYSTEM_HEADER_H
